@@ -27,10 +27,10 @@ pd.set_option('display.max_rows', 100)
 ## COMMAND LINE INTERFACE
 
 @click.group()
-@click.version_option("77.7.1", package_name="alacorder")
+@click.version_option("77.8.1", package_name="alacorder")
 def cli():
     """
-    ALACORDER beta 77.7
+    ALACORDER beta 77.8
 
     Alacorder retrieves case detail PDFs from Alacourt.com and processes them into text archives and data tables suitable for research purposes.
 
@@ -299,6 +299,58 @@ def fetch(listpath, path, cID, uID, pwd, qmax, qskip, speed, no_log, no_update,
             query_writer['CASES_FOUND'][n] = str(len(results))
             query_writer.to_excel(listpath,sheet_name="PartySearchQuery",index=False)
 
+@cli.command(help="Mark query template sheet with cases found in archive or PDF directory input")
+@click.option("--input-path", "-in", "in_path", required=True, prompt="Path to archive / PDF directory", help="Path to query template spreadsheet (.csv, .xls(x), .json)", type=click.Path())
+@click.option("--output-path", "-out", "out_path", required=True, prompt="Query template spreadsheet path", type=click.Path(), help="Path to output query template spreadsheet")
+@click.option('--no-write','-n', default=False, is_flag=True, help="Do not export to output path", hidden=True)
+def mark(in_path, out_path, no_write=False):
+
+    # get input text, names, dob
+    input_archive = alac.read(in_path)
+    mapinputs = alac.setinputs(input_archive)
+    mapoutputs = alac.setoutputs()
+    mapconf = alac.set(mapinputs, mapoutputs, no_write=True, no_prompt=True, overwrite=True, log=False, debug=True)
+
+    caseinfo = alac.map(mapconf, lambda x: x, alac.getCaseNumber, alac.getName, alac.getDOB, names=['TEXT','CASE','NAME','DOB'])
+
+    # get output cols 
+    output_query = alac.readPartySearchQuery(out_path)[0]
+
+    # get common columns
+    q_columns = pd.Series(output_query.columns).astype("string")
+    i_columns = pd.Series(caseinfo.columns).astype("string")
+    q_columns = q_columns.str.upper().str.strip().str.replace(" ","_")
+    i_columns = i_columns.str.upper().str.strip().str.replace(" ","_")
+    common = q_columns.map(lambda x: x in i_columns.tolist())
+    common_cols = q_columns[common]
+
+    assert common_cols.shape[0] > 0
+
+    output_query['RETRIEVED_ON'] = output_query.index.map(lambda x: time.time() if str(output_query.NAME[x]).replace(",","") in caseinfo.NAME.tolist() and output_query.RETRIEVED_ON[x] == "" else output_query.RETRIEVED_ON[x])
+    output_query['CASES_FOUND'] = output_query['CASES_FOUND'].map(lambda x: pd.to_numeric(x))
+    output_query['RETRIEVED_ON'] = output_query['RETRIEVED_ON'].map(lambda x: pd.to_numeric(x))
+    if not no_write:
+        with pd.ExcelWriter(out_path) as writer:
+            output_query.to_excel(writer, sheet_name="MarkedQuery", engine="openpyxl")
+
+    return output_query
+
+
+@cli.command(help="Append one case text archive to another")
+@click.option("--input-path", "-in", "in_path", required=True, prompt="Path to archive / PDF directory", help="Path to input archive", type=click.Path())
+@click.option("--output-path", "-out", "out_path", required=True, prompt="Query template spreadsheet path", type=click.Path(), help="Path to output archive")
+@click.option('--no-write','-n', default=False, is_flag=True, help="Do not export to output path", hidden=True)
+def append(in_path, out_path, no_write=False):
+    input_archive = alac.read(in_path)
+    output_archive = alac.read(out_path)
+    new_archive = pd.concat([output_archive, input_archive], ignore_index=True)
+    if not no_write:
+        cin = alac.setinputs(input_archive)
+        cout = alac.setoutputs(out_path)
+        conf = alac.set(cin, cout)
+        alac.write(conf, new_archive)
+    return new_archive
+
 
 if __name__ == "__main__":
     cli()
\ No newline at end of file
