@@ -38,6 +38,55 @@ warnings.filterwarnings('ignore')
 
 ## CONFIG
 
+def append_archive(in_path, out_path, no_write=False):
+    input_archive = read(in_path)
+    output_archive = read(out_path)
+    new_archive = pd.concat([output_archive, input_archive], ignore_index=True)
+    if not no_write:
+        cin = setinputs(input_archive)
+        cout = setoutputs(out_path)
+        conf = set(cin, cout)
+        alac.write(conf, new_archive)
+    return new_archive
+
+def read(path, log=True):
+   if os.path.isdir(path):
+      pdfpaths = pd.Series(glob.glob(path + '**/*.pdf', recursive=True))
+      if log:
+         tqdm.pandas(desc="PDF=>Text")
+         pdftexts = pdfpaths.progress_map(lambda x: getPDFText(x))
+      else:
+         pdftexts = pdfpaths.map(lambda x: getPDFText(x))
+      archive = pd.DataFrame({
+         'Timestamp': time.time(),
+         'AllPagesText': pdftexts,
+         'Path': pdfpaths
+         })
+      archive.AllPagesText = archive.AllPagesText.astype("string")
+      assert len(pdfpaths) > 0
+      return archive
+   else:
+      ext = os.path.splitext(path)[1]
+      nzext = os.path.splitext(path.replace(".zip","").replace(".xz",""))[1]
+      if nzext == ".pkl" and ext == ".xz":
+         archive = pd.read_pickle(path, compression="xz")
+      if nzext == ".pkl" and ext == ".pkl":
+         archive = pd.read_pickle(path)
+      elif nzext == ".json" and ext == ".zip":
+         archive = pd.read_json(path, orient='table', compression="zip")
+      elif nzext == ".json" and ext == ".json":
+         archive = pd.read_json(path, orient='table')
+      elif nzext == ".csv" and ext == ".zip":
+         archive = pd.read_csv(path, compression="zip")
+      elif nzext == ".csv" and ext == ".csv":
+         archive = pd.read_csv(path, compression="zip")
+      elif nzext == ".parquet" and ext == ".zip":
+         archive = pd.read_parquet(path,compression="zip")
+      elif nzext == ".parquet" and ext == ".parquet":
+         archive = pd.read_parquet(path)
+      assert "AllPagesText" in archive.columns
+      return archive
+
 def setinputs(path, debug=False, fetch=False):
    """Verify and configure input path. Must use set() to finish configuration even if NO_WRITE mode. Call setoutputs() with no arguments.  
    
@@ -57,19 +106,19 @@ def setinputs(path, debug=False, fetch=False):
          ECHO: (IFrame(str)) log data for console 
       })
    """
-   if fetch == True or (os.path.splitext(path)[1] in [".xlsx",".xls",".csv",".json"]):
-      queue = readPartySearchQuery(path)
-      out = pd.Series({
-         'INPUT_PATH': path,
-         'IS_FULL_TEXT': False,
-         'QUEUE': queue,
-         'FOUND': 0,
-         'GOOD': True,
-         'PICKLE': '',
-         'ECHO': ''
-      })
-      return out
-      
+   if fetch == True and isinstance(path, str) and not isinstance(path, pd.core.frame.DataFrame) and not isinstance(path, pd.core.series.Series):
+      if fetch == True or (os.path.splitext(path)[1] in [".xlsx",".xls",".csv",".json"]):
+         queue = readPartySearchQuery(path)
+         out = pd.Series({
+            'INPUT_PATH': path,
+            'IS_FULL_TEXT': False,
+            'QUEUE': queue,
+            'FOUND': 0,
+            'GOOD': True,
+            'PICKLE': '',
+            'ECHO': ''
+         })
+         return out
    else:
       found = 0
       is_full_text = False
@@ -1336,9 +1385,12 @@ def getPDFText(path: str) -> str:
       str: Description
    """
    text = ""
-   pdf = PyPDF2.PdfReader(path)
-   for pg in pdf.pages:
-      text += pg.extract_text()
+   try:
+      pdf = PyPDF2.PdfReader(path)
+      for pg in pdf.pages:
+         text += pg.extract_text()
+   except:
+      text = ""
    return text
 
 def getCaseNumber(text: str):
@@ -1937,6 +1989,38 @@ def getCharges(text):
 
 ## FETCH
 
+def mark(in_path, out_path, no_write=False):
+
+    # get input text, names, dob
+    input_archive = read(in_path)
+    mapinputs = setinputs(input_archive)
+    mapoutputs = setoutputs()
+    mapconf = set(mapinputs, mapoutputs, no_write=True, no_prompt=True, overwrite=True, log=False, debug=True)
+
+    caseinfo = map(mapconf, lambda x: x, getCaseNumber, getName, getDOB, names=['AllPagesText','CaseNumber','NAME','DOB'])
+
+    # get output cols 
+    output_query = readPartySearchQuery(out_path)[0]
+
+    # get common columns
+    q_columns = pd.Series(output_query.columns).astype("string")
+    i_columns = pd.Series(caseinfo.columns).astype("string")
+    q_columns = q_columns.str.upper().str.strip().str.replace(" ","_")
+    i_columns = i_columns.str.upper().str.strip().str.replace(" ","_")
+    common = q_columns.map(lambda x: x in i_columns.tolist())
+    common_cols = q_columns[common]
+
+    assert common_cols.shape[0] > 0
+
+    output_query['RETRIEVED_ON'] = output_query.index.map(lambda x: time.time() if str(output_query.NAME[x]).replace(",","") in caseinfo.NAME.tolist() and output_query.RETRIEVED_ON[x] == "" else output_query.RETRIEVED_ON[x])
+    output_query['RETRIEVED_ON'] = output_query['RETRIEVED_ON'].map(lambda x: pd.to_numeric(x))
+    output_query['CASES_FOUND'] = output_query['CASES_FOUND'].map(lambda x: pd.to_numeric(x))
+    if not no_write:
+        with pd.ExcelWriter(out_path) as writer:
+            output_query.to_excel(writer, sheet_name="MarkedQuery", engine="openpyxl")
+
+    return output_query
+
 def fetch(listpath, path, cID, uID, pwd, qmax=0, qskip=0, speed=1, no_log=False, no_update=False, debug=False):
    """
    Use headers NAME, PARTY_TYPE, SSN, DOB, COUNTY, DIVISION, CASE_YEAR, and FILED_BEFORE in an Excel spreadsheet to submit a list of queries for Alacorder to fetch.
@@ -2006,7 +2090,8 @@ def fetch(listpath, path, cID, uID, pwd, qmax=0, qskip=0, speed=1, no_log=False,
       if not no_update:
          query_writer['RETRIEVED_ON'][n] = str(math.floor(time.time()))
          query_writer['CASES_FOUND'][n] = str(len(results))
-         query_writer = query_writer.convert_dtypes()
+         query_writer['RETRIEVED_ON'] = query_writer['RETRIEVED_ON'].map(lambda x: pd.to_numeric(x))
+         query_writer['CASES_FOUND'] = query_writer['CASES_FOUND'].map(lambda x: pd.to_numeric(x))
          query_writer.to_excel(listpath,sheet_name="PartySearchQuery",index=False)
    return [driver, query_writer]
 
@@ -2046,7 +2131,7 @@ def party_search(driver, name = "", party_type = "", ssn="", dob="", county="",
    Returns:
       URL list to PDFs
    """
-   speed = speed * 1.5
+   speed = speed * 2
 
 
    if "frmIndexSearchForm" not in driver.current_url:
@@ -2175,7 +2260,7 @@ def party_search(driver, name = "", party_type = "", ssn="", dob="", county="",
             continue
    return pdflinks
 
-def downloadPDF(driver, url, no_log=False, cID="", uID="", pwd=""):
+def downloadPDF(driver, url, no_log=False, cID="", uID="", pwd="", speed=2):
    """
    With (driver), download PDF at (url)
    
@@ -2303,11 +2388,11 @@ def readPartySearchQuery(path, qmax=0, qskip=0, speed=1, no_log=False):
 
    clist = []
    for c in query.columns:
-      if c.upper().strip().replace(" ","_") in ["NAME", "PARTY", "DATE_OF_BIRTH", "BIRTHDATE", "PARTY_TYPE", "SSN", "DOB", "COUNTY", "DIVISION", "CASE_YEAR", "NO_RECORDS", "FILED_BEFORE", "FILED_AFTER", "RETRIEVED_ON", "CASES_FOUND"]:
-         ce = c.replace("DATE_OF_BIRTH","DOB").replace("BIRTHDATE","DOB").replace("PARTY","PARTY_TYPE").replace("PARTY_TYPE_TYPE","PARTY_TYPE").strip()
+      if str(c).upper().strip().replace(" ","_") in ["NAME", "PARTY", "DATE_OF_BIRTH", "BIRTHDATE", "PARTY_TYPE", "SSN", "DOB", "COUNTY", "DIVISION", "CASE_YEAR", "NO_RECORDS", "FILED_BEFORE", "FILED_AFTER", "RETRIEVED_ON", "CASES_FOUND"]:
+         ce = str(c).replace("DATE_OF_BIRTH","DOB").replace("BIRTHDATE","DOB").replace("PARTY","PARTY_TYPE").replace("PARTY_TYPE_TYPE","PARTY_TYPE").strip()
          clist += [ce]
-         query_out[c.upper().strip().replace(" ","_")] = query[c]
-         query_out[ce] = query[c]
+         query_out[str(c).upper().strip().replace(" ","_")] = query[str(c)]
+         query_out[ce] = query[str(c)]
    clist = pd.Series(clist).drop_duplicates().tolist()
    if clist == []:
       raise Exception("Invalid template! Use headers NAME, PARTY_TYPE, SSN, DOB, COUNTY, DIVISION, CASE_YEAR, and FILED_BEFORE in a spreadsheet or JSON file to submit a list of queries for Alacorder to fetch.")
@@ -2318,6 +2403,7 @@ def readPartySearchQuery(path, qmax=0, qskip=0, speed=1, no_log=False):
 
 ## LOGS
 
+
 def echo_conf(input_path, make, output_path, overwrite, no_write, dedupe, no_prompt, compress):
    """
    Logs configuration details to console
